/*
 * $LastChangedRevision: 2722 $
 * $LastChangedDate:: 2022-10-25 16:11:21 +0200#$
 *
 * This file is part of X2C. https://x2c.lcm.at/
 *
 * Copyright (c) 2019, Linz Center of Mechatronics GmbH (LCM) http://www.lcm.at/
 * All rights reserved.
 */
/*
 * This file is licensed according to the BSD 3-clause license as follows:
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the "Linz Center of Mechatronics GmbH" and "LCM" nor
 *       the names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL "Linz Center of Mechatronics GmbH" BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "LNet.h"
#include "Services.h"
#include "BlockServices.h"
#include "VersionInfo.h"
#include "X2C.h"
#include "GlobalDefines.h"
#include "Hardware.h"
#include "InputControl.h"
#include "OutputControl.h"
#include "Main.h"


/* X2C maintenance structure */
volatile tTableStruct TheTableStruct;
volatile tTableStruct *TableStruct = &TheTableStruct;

/* LNet protocol buffersize & node identifier */
#define LNET_BUFFERSIZE ((uint16)255)
#define LNET_NODEID ((uint8)1)

static tSerial interface;
static tLNet protocol;
static uint8 bufferLNet[LNET_BUFFERSIZE];


/** 
 * @brief Main function.
 * 
 * @return The main function will never return due to the never ending loop.
 */
int main(void)
{
    /** - initialize "integrated bootloader":
     *    - configuration of LNet protocol:
     *      - Node-ID: 1
     *      - Buffer size: 255
     *      .
     *    .
     *  - initialize serial interface
     */
    initTableStruct();
    initSerialGeneric(&interface);
    initLNet(&protocol, bufferLNet, LNET_BUFFERSIZE, LNET_NODEID);
    initServiceTable((tProtocol*)&protocol);
    addCoreServices((tProtocol*)&protocol);
    addBlockServices((tProtocol*)&protocol);
    addTableStructProtocol((tProtocol*)&protocol);
    linkSerial((tProtocol*)&protocol, &interface);
    initSerial(&interface);
    TableStruct->DSPState = PRG_LOADED_STATE;

    initVersionInfo(TableStruct, (uint16)1);   /* software version 1 */

    /** - initialize X2C */
    X2C_Init();
    KICK_DOG;
    
    /** - initialize hardware */
    initHardware();
    KICK_DOG;
    
    /** - initialize CPU load measurement */
    CPU_Scale = X2C_SAMPLETIME;

    /** - never ending loop -> interrupt driven algorithm */
    while (1)
    {
        /* serial communication */
        protocol.communicate((tProtocol*)&protocol);
        
        /* kick watchdog */
        KICK_DOG;
    }
    return (0);
}

/** 
 * @brief Main control task.
 *
 * This task is/has to be called periodically.
 * Calling rate = Sample time defined in GlobalDefines.h
 */
void mainTask(void)
{
    /*------------------------------------------------------------------------*/
    /** - assign inports */
    /*------------------------------------------------------------------------*/
    readAnalogIn();
    readDigitalIn();
   
    /*------------------------------------------------------------------------*/
    /** - update X2C */
    /*------------------------------------------------------------------------*/
    X2C_Update();

    /*------------------------------------------------------------------------*/
    /** - update outports */
    /*------------------------------------------------------------------------*/
    setPWM();
    writeDigitalOut();
}
